%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ejemplo de programacion de un entorno de instrumentacion.
% Programador: Jose M. Drake
% Fecha: 16-11-06
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se eliminan todos los objetos Visa remanetes de ejecuciones previas  
instrreset;
% Se eliminan todas las variables del entorno MatLab
clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constantes
R0= 8200;  % Resistencia en serie
NT= 1000;  % Puntos por ciclos en la forma de onda
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se crea y configura el objeto Visa-gpib para controlar el generador
% El HPIB-Address del generador es 10. 
generador=visa('agilent','GPIB0::10::0::INSTR');
generador.Timeout=2.0;
generador.EOIMode='on';
generador.EOSMode='none'
% Se establece la conexion con el equipo.
fopen(generador);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se crea un objeto Visa-gpib para controlar el osciloscopio
% El HPIB-Address del generador es 5. 
oscilo=visa('agilent','GPIB0::5::0::INSTR');
oscilo.Timeout=2.0;
oscilo.EOIMode='on';
oscilo.EOSMode='none'
oscilo.inputBufferSize=4256;
% Se establece la conexion con el equipo.
fopen(oscilo);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se establece el generador para que genere la seal sinusoidal de:
%  5 vpp de amplitud, 200 Hz de frecuencia y offset nulo
%  
fprintf(generador,'*CLS');
fprintf(generador,'*RST');
%fprintf(generador,'SYSTEM:REMOTE');
fprintf(generador,'OUTPUT:Load INFINITY');
fprintf(generador,'APPLY:Sinusoid 200 Hz,5 vpp, 0 v');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Se establece el oscilador para que genere la seal establecida en el  
fprintf(oscilo,'*CLS');
fprintf(oscilo,'*RST');
fprintf(oscilo,'System:LOCK on');
% Inicialza valores de canal 1
fprintf(oscilo,'CHANNEL1:Coupling DC');
fprintf(oscilo,'CHANNEL1:Offset 0.0');
fprintf(oscilo,'CHANNEL1:Probe x10');
fprintf(oscilo,'CHANNEL1:Range 10.0');
fprintf(oscilo,'VIEW Channel1');
% Inicialza valores de canal 2
fprintf(oscilo,'CHANNEL2:Coupling DC');
fprintf(oscilo,'CHANNEL2:Offset 0.0');
fprintf(oscilo,'CHANNEL2:Probe x10');
fprintf(oscilo,'CHANNEL2:Range 5.0');
fprintf(oscilo,'VIEW Channel2');
% Estable la base tiempos del osciloscopio
fprintf(oscilo,'TIMEBASE:Range 10.0 ms');
fprintf(oscilo,'TRIGGER:Mode AUTLEVEL');
fprintf(oscilo,'TRIGGER:Source EXTERNAL');
pause(2);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comienza la primera medida
input('Pulse Return para continuar...');
%
% Requiere del osciloscopio el valor rms y la fase del canal #2
fprintf(oscilo,'MEASURE:SOURCE CHANNEL2');
fprintf(oscilo,'MEASURE:VRMS');
fprintf(oscilo,'MEASURE:VRMS?');
StrVZrms=fscanf(oscilo);
VZrms=str2num(StrVZrms);
fprintf(oscilo,'MEASURE:Period');
fprintf(oscilo,'MEASURE:Period?');
StrT=fscanf(oscilo);
T=str2num(StrT);
fprintf(oscilo,'MEASURE:TVolt? 0.0,+1,Channel2');
pause(1);
T2=str2num(fscanf(oscilo));
F2=2*pi*T2/T;
VZCompleja=VZrms*exp(j*F2);
display(strcat('Vz: Ampl.rms= ',StrVZrms,' Vrms   Fase=',num2str(F2*180/pi),'    Tension compleja= ',num2str(VZCompleja)));
% Requiere del osciloscopio el valor de continua del canal #1
fprintf(oscilo,'MEASURE:SOURCE CHANNEL1');
fprintf(oscilo,'MEASURE:VRMS');
fprintf(oscilo,'MEASURE:VRMS?');
StrVSrms=fscanf(oscilo);
VSrms=str2num(StrVSrms);
fprintf(oscilo,'MEASURE:TVolt? 0.0,+1,Channel1');
T1=str2num(fscanf(oscilo));
F1=2*pi*T1/T;
VSCompleja=VSrms*exp(j*F1);
display(strcat('Vs: Ampl.rms= ',StrVSrms,' Fase=',num2str(F1*180/pi),'   Tension compleja= ',num2str(VSCompleja)));
%
%Calculo de las potencias activa y reactiva
% 
PCompleja=VZCompleja*(VSCompleja-VZCompleja)/R0;
PActiva=real(PCompleja);
PReactiva=imag(PCompleja)
display(strcat('Potencia Activa= ', num2str(PActiva),' W    Potencia reactiva= ',num2str(PReactiva),' W'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comienza la segunda medida
input('Pulse Return para continuar...');
%
% Define las caracteristicas de codificacion de las formas de onda
fprintf(oscilo,'WAVEFORM:Format BYTE');
fprintf(oscilo,'WAVEFORM:Source CHANNEL1');
fprintf(oscilo,strcat('WAVEFORM:Points 2000'));
fprintf(oscilo,'ACQUIRE:Type NORMAL');
fprintf(oscilo,'ACQUIRE:Complete 100');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Requiere del osciloscopio la forma de onda
fprintf(oscilo,'DIGITIZE Channel1,Channel2');

fprintf(oscilo,'WAVEFORM:Source CHANNEL2');
fprintf(oscilo,'WAVEFORM:DATA?');
CodVz=binblockread(oscilo);

fprintf(oscilo,'WAVEFORM:XINCREMENT?');
xincrement=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Xorigin?');
xorigin=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Xreference?');
xreference=str2num(fscanf(oscilo));
% Tiempos en segundo de cada muestra
t=((1:2000)-xreference)*xincrement+xorigin;

fprintf(oscilo,'WAVEFORM:Yincrement?');
yincrement=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Yorigin?');
yorigin=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Yreference?');
yreference=str2num(fscanf(oscilo));

% Valores de la tension en la carga en funcion del tiempo
Vz=(CodVz-yreference)*(yincrement)+yorigin;

fprintf(oscilo,'WAVEFORM:Source CHANNEL1');
fprintf(oscilo,'WAVEFORM:DATA?');
CodVs=binblockread(oscilo);

fprintf(oscilo,'WAVEFORM:Yincrement?');
yincrement=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Yorigin?');
yorigin=str2num(fscanf(oscilo));
fprintf(oscilo,'WAVEFORM:Yreference?');
yreference=str2num(fscanf(oscilo));

% Valores de la tension en la fuente en funcion del tiempo
Vs=(CodVs-yreference)*(yincrement)+yorigin;

% Valores de la intensidad en la carga
Iz=(Vs-Vz)/R0;
plot(t,Vz,'r',t,10000*Iz,'g');
% Calculo de la potencia activa
PTActiva=0;
PTReactiva=0;
for n=1:NT
    PTActiva=PTActiva+Vz(n)*Iz(n);
    PTReactiva=PTReactiva+Vz(n)*Iz(n+NT/4);
end;
PTActiva=PTActiva/NT;
PTReactiva=PTReactiva/NT;
display('Calculo de potencias basado en las formas de ondas en el dominio del tiempo');
display(strcat('Potencia Activa= ', num2str(PTActiva),' W    Potencia reactiva= ',num2str(PTReactiva),' W'));
fclose(generador);
fclose(oscilo);
delete(generador);
delete(oscilo);
clear oscilo;
clear generador;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end;